@@ -5,14 +5,14 @@ Created on Aug 2, 2013
 '''
 from contextlib import contextmanager
 from datetime import datetime
-from mtq.log import StreamHandler, MongoStream, MongoHandler
-from mtq.utils import handle_signals, setup_logging, now, is_py3
+from mtq.log import MongoStream, MongoHandler
+from mtq.utils import handle_signals, now, setup_logging2, nulltime
 from multiprocessing import Process
 import logging
 import os
 import sys
 import time
-import io
+
 
 class Worker(object):
     '''
@@ -20,9 +20,9 @@ class Worker(object):
     '''
     def __init__(self, factory, queues=(), tags=(), priority=0,
                  poll_interval=1, exception_handler=None,
-                 log_worker_output=False, silence=False):
-        
+                 log_worker_output=False, silence=False, extra_lognames=()):
         self.name = '%s.%s' % (os.uname()[1], os.getpid())
+        self.extra_lognames = extra_lognames
         
         self.queues = queues
         self.tags = tags
@@ -41,6 +41,7 @@ class Worker(object):
         self.silence = silence
         
         
+    worker_id = '-'
     
     @contextmanager
     def register(self):
@@ -55,26 +56,43 @@ class Worker(object):
         self.collection = self.factory.worker_collection
         
         self.worker_id = self.collection.insert({'name': self.name,
+                                                 'host': os.uname()[1],
+                                                 'pid': os.getgid(),
+                                                 'user': os.getlogin(),
                                                  'started':now(),
                                                  'finished':datetime.fromtimestamp(0),
+                                                 'check-in':datetime.fromtimestamp(0),
                                                  'working':True,
                                                  'queues': self.queues,
                                                  'tags': self.tags,
                                                  'log_output': bool(self._log_worker_output),
+                                                 'terminate': False,
+                                                 'terminate_status': 0,
                                                  })
-
-        hdlr = MongoHandler(self.factory.logging_collection, {'worker_id':self.worker_id})
-        self.logger.addHandler(hdlr)
+        if self._log_worker_output:
+            hdlr = MongoHandler(self.factory.logging_collection, {'worker_id':self.worker_id})
+            self.logger.addHandler(hdlr)
         try:
             yield self.worker_id
         finally:
-            self.logger.removeHandler(hdlr)
-            self.collection.update({'_id': self.worker_id},
-                                   {'$set':{'finished':now(),
-                                            'working':False
-                                            }
-                                    })
+            if self._log_worker_output:
+                self.logger.removeHandler(hdlr)
+                
+            query = {'_id': self.worker_id}
+            update = {'$set':{'finished':now(), 'working':False}}
+            self.collection.update(query, update)
+    
+    def check_in(self):
         
+        query = {'_id': self.worker_id}
+        update = {'$set':{'check-in':now(), 'working':True}}
+        worker_info = self.collection.find_and_modify(query, update)
+        if worker_info:
+            should_exit = worker_info.get('terminate', False)
+            status = worker_info.get('terminate_status', 0)
+            return should_exit, status
+        return False, 0
+
     def work(self, one=False, batch=False):
         '''
         Main work function
@@ -98,9 +116,16 @@ class Worker(object):
         '''
         Start the main loop and process jobs
         '''
+        self.logger.info('Starting Main Loop mogno-host=%s mongo-db=%s' % (self.factory.db.connection.host,
+                                                                           self.factory.db.name))
         self.logger.info('Starting Main Loop worker=%s _id=%s' % (self.name, self.worker_id))
         self.logger.info('Listening for jobs queues=[%s] tags=[%s]' % (', '.join(self.queues), ', '.join(self.tags)))
+        
         while 1:
+            should_exit, status = self.check_in()
+            if should_exit:
+                self.logger.info("Shutdown Requested (from DB)")
+                raise SystemExit(status)
             
             job = self.factory.pop_item(worker_id=self.worker_id,
                                         queues=self.queues,
@@ -152,29 +177,17 @@ class Worker(object):
         '''
         handle_signals()
         
-        if self._log_worker_output:
-            setup_logging(self.worker_id, job.id, self.factory.logging_collection, self.silence)
-        elif self.silence:
-            if is_py3():
-                sys.stderr = io.StringIO()
-                sys.stdout = io.StringIO()
-            else:
-                sys.stderr = io.BytesIO()
-                sys.stdout = io.BytesIO()
-        
-        logger = logging.getLogger('job')
-        logger.info('Starting Job %s' % job.id)
-        try:
-            self._pre(job)
-            job.apply()
-        except:
-            if self._handler:
-                exc_type, exc_value, traceback = sys.exc_info()
-                self._handler(job, exc_type, exc_value, traceback)
-            raise
-        finally:
-            self._post(job)
-                
+        with setup_logging2(self.worker_id, job.id, lognames=self.extra_lognames):
+            try:
+                self._pre(job)
+                job.apply()
+            except:
+                if self._handler:
+                    exc_type, exc_value, traceback = sys.exc_info()
+                    self._handler(job, exc_type, exc_value, traceback)
+                raise
+            finally:
+                self._post(job)
     
     def _pre(self, job):
         if self._pre_call: self._pre_call(job)
@@ -191,8 +204,15 @@ class Worker(object):
     def push_exception_handler(self, handler):
         self._handler = handler
     
-    
+    @property
+    def num_backlog(self):
+        'number of tasks this worker has to complete'
+        return self.factory._items_cursor(queues=self.queues,
+                                          tags=self.tags,
+                                          ).count()
+
 class WorkerProxy(object):
+    'This is a representation of an actual worker process'
     
     def __init__(self, factory, doc):
         self.factory = factory
@@ -216,9 +236,22 @@ class WorkerProxy(object):
         
     @property
     def num_processed(self):
-        collection = self.factory.worker_collection
+        'number of tasks this worker has completed'
+        collection = self.factory.queue_collection
         return collection.find({'worker_id': self.id}).count()
     
+    @property
+    def num_backlog(self):
+        'number of tasks this worker has to complete'
+        return self.factory._items_cursor(queues=self.qnames,
+                                          tags=self.tags,
+                                          ).count()
+    
+    @property
+    def last_check_in(self):
+        'last check in time'
+        return self.doc.get('check-in', nulltime())
+        
     def stream(self):
         collection = self.factory.logging_collection
         
@@ -227,6 +260,7 @@ class WorkerProxy(object):
                            finished=self.finished)
     
     def finished(self):
+        'test if this worker is finished'
         coll = self.factory.worker_collection
         cursor = coll.find({'_id':self.id, 'working':False})
         return bool(cursor.count())
